@@ -19,11 +19,14 @@
 from collections import defaultdict
 from collections import deque
 from collections import namedtuple
+import functools
+import importlib.resources
 import itertools
 import math
 import operator
+import string
 
-from turberfield.catchphrase.drama import Drama
+from turberfield.catchphrase.mediator import Mediator
 from turberfield.dialogue.model import Model
 from turberfield.dialogue.model import SceneScript
 from turberfield.dialogue.performer import Performer
@@ -35,12 +38,23 @@ class Presenter:
     Animation = namedtuple("Animation", ["delay", "duration", "element"])
 
     @staticmethod
-    def build_presenter(folder, /, *args, ensemble=[], strict=True, roles=1, **kwargs):
+    @functools.lru_cache()
+    def load_dialogue(pkg, resource):
+        """
+        FIXME: Docs
+        """
+        with importlib.resources.path(pkg, resource) as path:
+            return path.read_text(encoding="utf-8")
+
+    @staticmethod
+    def build_presenter(folder, data, facts=None, ensemble=None, strict=True, roles=1):
         rv = None
         for n, p in enumerate(folder.paths):
-            folder_dialogue = Drama.load_dialogue(folder.pkg, p)
-            text = Drama.write_dialogue(folder_dialogue, *args, **kwargs)
-            rv = Presenter.build_from_text(text, index=n, ensemble=ensemble, strict=strict, roles=roles, path=p)
+            text = Presenter.load_dialogue(folder.pkg, p)
+            text = string.Formatter().vformat(text, tuple([data or ""]), facts or defaultdict(str))
+            rv = Presenter.build_from_text(
+                text, index=n, ensemble=ensemble or [], strict=strict, roles=roles, path=p
+            )
             if rv:
                 break
 
@@ -52,8 +66,9 @@ class Presenter:
         selection = script.select(ensemble, roles=roles)
         if all(selection.values()) or (not strict and any(selection.values())):
             script.cast(selection)
+            casting = {next(iter(i.attributes.get("names", [])), None): i.persona for i in selection}
             model = script.run()
-            rv = Presenter(model, index=index, ensemble=ensemble, text=text)
+            rv = Presenter(model, index=index, casting=casting, ensemble=ensemble, text=text)
             for k, v in model.metadata:
                 rv.metadata[k].append(v)
             return rv
@@ -102,7 +117,7 @@ class Presenter:
                 continue
         return rv
 
-    def __init__(self, dialogue, index=None, scene=None, ensemble=None, text=""):
+    def __init__(self, dialogue, index=None, scene=None, casting=None, ensemble=None, text=""):
         self.index = index
         self.frames = [
             defaultdict(list, dict(
@@ -112,7 +127,8 @@ class Presenter:
             for i in getattr(dialogue, "shots", [])
             if scene is None or i.scene == scene
         ]
-        self.ensemble = ensemble
+        self.casting = casting or {}
+        self.ensemble = ensemble or []
         self.metadata = defaultdict(list)
         self.text = text
 
